#!/bin/csh -f
##############################################################################
## Author      # Avi Mittal
## Description # This script generates a orderly compilation of vhdl files 
##             # in a design
##             # It needs the following
##             #
##             # 1). vhdlalign.pl
##             #     This file pre conditions a vhdl file making it suitable
##             #     To be used by genDependencyList.pl
##             # 2). vhdlagrep.pl
##             #     This file extracts keywords from a vhdl file
##             #     and collects them in <*._aligned_greped> file to be
##             #     processed by genDependencyList.pl
##             #     The idea is to mke these files a lot shorter for
##             #     faster processing
##             # 3). genDependencyList.pl
##             #     This is the main script to extract dependencies and
##             #     put all the dependent file in a orderly fashion
##             # 4). dirs.cfg [optional list of directories to be searched]
## Example Use:# genCompileScript.csh <top entity file>
##############################################################################
## History     #
##############################################################################
##   Date   #   Who    # Version # Details
##############################################################################
## 24/04/08 #  Avi     #    1    # Initial Revision
##############################################################################

set title = `basename $0`; 
  if ($#argv == 0) then 
    set error = "Invalid number of arguments;At least One expected" 
    echo "ERROR ($title)   :" $error; 
    echo "usage> $title <top entity file>"
    exit 
  endif 
#cat ~mittal/DOCS/tele | grep -i ^$argv[1]


set unixname = `echo $HOME:t`
setenv PERL_SRC ./
setenv CSH_SRC ./
set del = "no"

if( -d $PERL_SRC ) then
else
  echo "ERROR! PERL_SRC directory $PERL_SRC does not exists or is not readable."
  exit
endif

#t#if( -d $CSH_SRC ) then
#t#else
#t#  echo "ERROR! CSH_SRC directory $CSH_SRC does not exists or is not readable."
#t#  exit
#t#endif



#check for dirs.cfg which is a file containing directories to be searched.
#if found, then for each directory in file, execute
#vhdlalign, and vhdlgrep on each *.vhd or *.vhdl file
if(-f dirs.cfg) then
  foreach ii (`cat dirs.cfg | grep -v \#`)
    if( -d $ii ) then
    else
      echo "ERROR! directory $ii does not exists or is not readable."
      echo "Check your dirs.cfg file"
      exit
    endif

    pushd $ii
    foreach jj (`ls *.vhd *.vhdl`)
      ${PERL_SRC}/vhdlalign.pl $jj
      ${PERL_SRC}/vhdlgrep.pl $jj\_aligned
      set del = "yes";
    end
    popd
  end
endif
#Now work in the current directory
foreach jj (`ls *.vhd *.vhdl`)
  ${PERL_SRC}/vhdlalign.pl $jj
  ${PERL_SRC}/vhdlgrep.pl $jj\_aligned
end

${PERL_SRC}/genDependencyList.pl $argv[1] .

rm -rf *.vhd*_aligned*

if($del == "yes") then
  foreach ii (`cat dirs.cfg | grep -v \#`)
    pushd $ii
      rm -rf *.vhd*_aligned*
    popd
  end
endif

echo "INFO: compile.scr Generated"
