#!/usr/bin/perl
##############################################################################
## Author      # Avi Mittal
## Description # This script generates the following
##             # 1). This script is capable of generating a order dependent 
##             #     compile script corresponding to a vhdl design
##             #     Inputs to the script: 
##             #     a).File containing top level entity 
##             #     b).Path of the directory to be searched for dependent
##             #        files
## History     #
##############################################################################
##   Date   #   Who    # Version # Details
##############################################################################
## 18/04/08 #  Avi     #    1    # Initial Revision
##############################################################################
## 18/04/08 #  Avi     #    2    # Now it can handle multiple directories
##############################################################################
## 23/04/08 #  Avi     #    3    # Now it requires preconditioning scripts
##                               # vhdlalign.pl and vhdlgrep.pl
##############################################################################
## 24/04/08 #  Avi     #    4    # Stopped messy printing, added output file
##                               # name as compile.scr
##############################################################################

use File::Find;

if($#ARGV < 1) {
  print "ERROR: insufficient fields\n";
  print "Usage:\nlinux_prompt> genDependencyList.pl <Top_Entity_File> <Search_Path>\n";
  exit;
}

if(!(-f @ARGV[0]))
{
  print "ERROR! Input file  @ARGV[0] does not exists. . . Exiting\n";
  exit;
}
if(!(-d @ARGV[1]))
{
  print "ERROR! Input path  @ARGV[1] does not exists. . . Exiting\n";
  exit;
}

open(out,">compile.scr") || die "Couldnft open mapfile for compile.scr\n" ;
push(@flist,@ARGV[0]);
push(@copyflist1,@ARGV[0]);

$iter = 0;
$DirScanned = 0;
foreach $ff (@flist)
{
  $lib_line = "";
  $use_line = "";
  $comp_line = "";
  $iter = $iter + 1;
  print "iter number = $iter\n";
  @NumberOfFiles[$iter] = 0;
  #f#print "operating on file#$ff#\n";
  open(infile0,"<$ff") || die "Couldnft open infile0. $ff\n ";
  #system("awk f/module/,/endmodule/f @ARGV[0]>@ARGV[0].module")
  while ($line0 = <infile0>) {

  #Lib Search Starts
    $line1 = $line0;
    chomp($line1);
    if($line1 =~ / library /i && $line1 =~ /;/) 
    {
      $line1 =~ s/ library / /ig;
      $line1 =~ s/;/ /g;
      $line1 =~ s/,/ /g;
      $lib_line = $lib_line.$line1;
      #f#print "lib_line is $lib_line\n";
    }

  #Lib Search Ends
  
  #Use Search Starts
  
    $line2 = $line0;
    chomp($line2);
    if($line2 =~ / use /i && $line2 =~ /;/) 
    {
      if(!($line2 =~ / configuration /i || $line2 =~ / entity /i))
      {
        $line2 =~ s/ use / /ig;
        $line2 =~ s/.all/ /ig;
        $line2 =~ s/;/ /g;
        $line2 =~ s/,/ /g;
        if(!($line2 =~ /ieee\./ig | $line2 =~ /std\./ig))
        {
          $use_line = $use_line.$line2;
        }
        #f#print "use_line is $use_line\n";
      }
    }
    $line3 = $line0;
    chomp($line3);
    if($line3 =~ / component /i && $line3 =~ /port/i) 
    {
      if(!($line3 =~ / end /i))
      {
        $comp_line = $comp_line.$line3;
      #print "comp_line is $comp_line\n";
      }
    }
  #Find Component Ends
  
  #Find Instances Starts
    $line4 = $line0;
    chomp($line4);
    if($line4 =~ / : / && !($line4 =~ /;/)) 
    {
      if($line4 =~ /(\w+)(\s+)(:)(\s+)(\w+)/)
      {
        $inst_name = $5;
        $inst_name =~ tr/A-Z/a-z/;
        push(@inst_list,$5); 
        #$inst_list[$iter] = $inst_list[$iter]." ".$5;
      }
    }# if($line4 =~ / : / && !($line4 =~ /;/)) 
  #Find Instances Ends
  #Find Entity Starts
    $line5 = $line0;
    chomp($line5);
    #Review the following, because I am using entity search only for iter 1
    if($line5 =~ /(entity)(\s+)(\w+)(\s+)(is)/i && $iter == 1)  
    {
        $entity_name = $3;
        $entity_name =~ tr/A-Z/a-z/;
        push(@inst_list,$entity_name);
        #f#print "Found entity = @inst_list\n";
    }# if($line5 =~ / : / ) 
  #Find Entity Ends
  } #while ($line0 = <infile0>)
    #print "final lib_line = $lib_line\n";
    @llist = split(/ /, $lib_line);
  
    $use_line =~ s/(\s+)/ /g;
    $use_line =~ s/^(\s+)//g;
    #f#print "use_line = $use_line\n";
    @ulist = split(/ /, $use_line);
    #f#print "final ulist = @ulist\n";
  
    foreach $libdotpkg (@ulist)
    {
      @pkgs = split(/\./,$libdotpkg);
      push(@pkg_list,$pkgs[1]);
    }
  
  
    $comp_line =~ s/component\s+/component\./ig;
    #print "comp_line = $comp_line\n";
    @eles = split(/ /,$comp_line);
    foreach $ele (@eles)
    {
      if($ele =~ /component/i)
      {
        @comp_and_name = split(/\./,$ele);
        push(@comp_list,$comp_and_name[1]);
      }
    }
  
    #f#print "lib_list = @llist\n";
    #f#print "pkg_list = @pkg_list\n";
    #f#print "comp_list = @comp_list\n";
    #f#print "inst_list = @inst_list\n";
  
  #@files = <*>;
  #foreach $file (@files) {
  #  print $file . "\n";
  #} 
  #$dir = "/users/amittal/misc/cadence_flow/DESIGN/itb/vhdl";
  #$dir = "/projects/morilla/MORILLA.Fprj/workAreas.Folder/amittal.Work/ITB/VHDL";
  $dir = "$ARGV[1]";
  #print "Current Dir = $dir\n";
  if($iter == 1)
  {
    push(@dirs,$ARGV[1]);
  }
  if(-f "./dirs.cfg" && $iter == 1)
  {
    print "Found a list of directories to search\n";
    open(DINP,"<./dirs.cfg");
      while($dlines0 = <DINP>) 
      {
        chomp($dlines0);
        $dlines0 =~ s/(\s+)//g;
        if($dlines0 =~ /\-\-.*/)
        {
          next;
        }
        if($dlines0 =~ /#.*/)
        {
          next;
        }
        if(-d $dlines0)
        {
          push(@dirs,$dlines0);
        }
        else
        {
          print "WARN! the directory $dlines0 doesnt exists, edit dirs.cfg\n";
        }
      }
    close(DINP);
  }# if(-f "./dirs.cfg" && $iter == 1)
  
  
  if($DirScanned == 0) 
  {
    foreach $dir (@dirs)
    {
      print "Searching Directory $dir\n";
      find(\&Wanted, $dir);
      $DirScanned = 1;
    }
  #print "file list = @files\n";
  }# if($DirScanned == 0) 

  
  #f#print "merging#@inst_list#@pkg_list#,iter=$iter\n";
  push(@inst_list,@pkg_list);
  #f#print "instance list before foreach ii = @inst_list\n";
  foreach $ii (@inst_list)
  {
    $found_configuration = 0;
    foreach $jj (@files)
    {
      open(INP,"<$jj");
      while($lines0 = <INP>) 
      {
        #chomp($lines0);
        $lines0 =~ s/\-\-.*//g;
        if($lines0 =~ /(configuration)(\s+)(\w+)(\s+)(of)(\s+)($ii)(\s+)(is)/i)
        {
          #f#print $lines0;
          push(@vhdlist,$jj);
          @NumberOfFiles[$iter] = @NumberOfFiles[$iter] + 1;
          $found_configuration = 1;
          last; # end the search now
        }
      }#while
      close(INP);
    }# foreach $jj (@files)
    if($found_configuration == 0)
    {
      #f#print "PROBLEM: for $ii, configuration was found\n";
    }
  }# foreach $ii (@inst_list)
  foreach $ii (@inst_list)
  {
    $found_architecture = 0;
    foreach $jj (@files)
    {
      open(INP,"<$jj");
      while($lines0 = <INP>) 
      {
        if($lines0 =~ /(architecture)(\s+)(\w+)(\s+)(of)(\s+)($ii)(\s+)(is)/i)
        {
          #print $lines0;
          push(@vhdlist,$jj);
          @NumberOfFiles[$iter] = @NumberOfFiles[$iter] + 1;
          $found_architecture = 1;
          last; # end the search now
        }
      }#while
      close(INP);
    }# foreach $jj (@files)
    if($found_architecture == 0)
    {
      #f#print "PROBLEM: for $ii, no entity, architecture or package was found\n";
    }
  }# foreach $ii (@inst_list)
  foreach $ii (@inst_list)
  {
    $found_entity = 0;
    foreach $jj (@files)
    {
      open(INP,"<$jj");
      while($lines0 = <INP>) 
      {
        $lines0 =~ s/\-\-.*//g;
        if($lines0 =~ /(entity)(\s+)($ii)(\s+)(is)/i)
        {
          #f#print $lines0;
          push(@vhdlist,$jj);
          @NumberOfFiles[$iter] = @NumberOfFiles[$iter] + 1;
          $found_entity = 1;
          last; # end the search now
        }
      }#while
      close(INP);
    }# foreach $jj (@files)
    if($found_entity == 0)
    {
      #f#print "PROBLEM: for $ii, no entity was found\n";
    }
  }# foreach $ii (@inst_list)
  foreach $ii (@inst_list)
  {
    $found_package = 0;
    foreach $jj (@files)
    {
      open(INP,"<$jj");
      while($lines0 = <INP>) 
      {
        $lines0 =~ s/\-\-.*//g;
        if($lines0 =~ /(package)(\s+)($ii)(\s+)(is)/i)
        {
          #f#print $lines0;
          push(@vhdlist,$jj);
          @NumberOfFiles[$iter] = @NumberOfFiles[$iter] + 1;
          $found_package = 1;
          last; # end the search now
        }
      }#while
      close(INP);
    }# foreach $jj (@files)
    if($found_package == 0)
    {
      #f#print "PROBLEM: for $ii, package was found\n";
    }
  }# foreach $ii (@inst_list)
  #f#print "vhdlist=@vhdlist\n";
  #f#print "flist is before push=@flist#\n\n";
  #f#print "number of files at the end of $iter = @NumberOfFiles\n";
  
  @copyflist = @flist;
  #@copyflist1 = @flist;
  @copyvhdlist = @vhdlist;
  push(@flist,@vhdlist);

  #f#print "flist is after push=@flist#\n\n";
  if($iter == 1) 
  {
    #push(@flist,@vhdlist);
    #push(@flist,$vhdlist[0]);
    @inst_list_1 = @inst_list;
    @vhdlist_1 = @vhdlist;
  }
  @inst_list = ();
  @vhdlist = ();
  @llist = ();
  @ulist = ();
  @comp_list = ();
  @pkg_list = ();
  
}# foreach $ff (@flist)

 #f#print "flist got = @flist, number of elements = $#flist\n";

while (@flist)
{
  $vhdlFile = pop(@flist);
  $found = 0;
  foreach $f1 (@flistNew)
  {
    if($vhdlFile eq $f1)
    {
      $found = 1;
    }
  }
  if($found == 0)
  {
    push(@flistNew,$vhdlFile);
  }
  #f#print "$vhdlFile\n";
}

pop(@flistNew); #Remove the first file, as it is now redundant
foreach $f1 (@flistNew)
{
  $f1 =~ s/_aligned_greped//g;
  print out "ncvhdl -work libname $f1\n";
}

#Subprogram declaration Section Starts

sub Wanted
{
  # only operate on .vhd or.vhdl files
  /greped/ or return;	
  push(@files,$File::Find::name);
}

