#!/usr/bin/perl
##############################################################################
## Author      # Avi Mittal
## Description # This script generates synth constraints for any vhdl file
               # The idea is to generate input and output constraints for
               # each input and output port in a design
               # It puts a default value of 0.7*clk_period on each port
               # of the design, which gives about 0.4*clk_period for the 
               # signals to propogate from one entity to other, if both
               # entities have had their constraiints generated using this
               # script
##############################################################################
##   Date   #   Who    # Version # Details
##############################################################################
## 24/04/08 #  Avi     #    1    # Initial Revision
##############################################################################
## 24/04/08 #  Avi     #    2    # 1.Corrected date on the header above
##                     #         # 2.More robust in handling
##############################################################################
## 01/05/08 #  Avi     #    3    # 1.Added false paths templates
##############################################################################

use File::Find;


if($#ARGV < 0) {
  print "ERROR: insufficient fields\n";
  print "Usage:\nlinux_prompt> vhdlgrep.pl <vhdl file> [clk name]\n";
  exit;
}

if(!(-f @ARGV[0]))
{
  print "ERROR! Input file  @ARGV[0] does not exists. . . Exiting\n";
  exit;
}

$ff = "$ARGV[0]";
@ffs = split(/\//, $ff);
$out_file_name = pop(@ffs);
$found_entity = 0;
$in_port_scaler = zero;
$out_port_scaler = zero;

$clk_name = clk_viterbi;
if($#ARGV == 1)
{
  $clk_name = @ARGV[1];
}

#f#print "file name minus path = $out_file_name,array = @ffs,orig argv0 = $ff\n";
$line_number = 0;
open(out,">constraints.tcl") || die "Could not open mapfile for constraints.tcl\n" ;
  open(infile0,"<$ff") || die "Couldnft open infile0. $ff\n ";
  #system("awk f/module/,/endmodule/f @ARGV[0]>@ARGV[0].module")
  while ($line0 = <infile0>) 
  {
    chomp($line0);
    $line_number ++;
    $line0 =~ s/^/ /g;          #add a space at start
    $line0 =~ s/$/ /g;          #add a space at end
    $line0 =~ s/\-\-.*//g;      #remove comments
    $line0 =~ s/;/ ; /g;        #add space
    $line0 =~ s/\(/ \( /g;      #add space
    $line0 =~ s/\)/ \) /g;      #add space
    $line0 =~ s/,/ , /g;        #add space
    $line0 =~ s/:/ : /g;        #add space
    $line0 =~ s/</ < /g;        #add space
    $line0 =~ s/< =/<=/g;       #remove space
    $line0 =~ s/>/ > /g;        #add space
    $line0 =~ s/= >/=>/g;       #remove space
    $line0 =~ s/\t+/ /g;        #No tabs wanted
    $line0 =~ s/\s+/ /g;        #Only single spaces wanted
    if(!($line0 =~ /\S/))       #Skip empty lines
    {
      next;
    }
    if($line0 =~ /( entity )/i)
    {
      $found_entity = $found_entity + 1;
      #$entity_name = $entity_name." ".$3;
      #print "Entity found = $entity_name\n";
    }
    if($line0 =~ / end /i)
    {
      $found_end = 1;
      #print "End found\n";
      $port_line = $port_line." ".$line0;
      last;
    }
    $port_line = $port_line." ".$line0;
    #print "$port_line\n";
    #print "line_number = $line_number\n";
  } #while ($line0 = <infile0>)
    #print "$port_line\n,found_entity = $found_entity,found_end = $found_end\n";

  if($found_entity == 0)
  {
    print "ERROR! No 'entity' declaration found in the given file\n";
    exit;
  }

  if(!($port_line =~ /(.*)(entity)(\s+)(\w+)(\s+)(is )/i))
  {
    $found_entity = 0;
    print "$port_line\n,found_entity = $found_entity,found_end = $found_end\n";
  }
  else
  {
    print "Name of Entity found = $4\n";
  }

  
  if($found_entity == 0)
  {
    print "ERROR! No 'entity' declaration found in the given file\n";
    exit;
  }

  if($found_entity == 1 && $found_end == 1)
  {
    if($port_line =~ /(.*)( port )(.*)( end )/i)
    {
      #print "$3\n";
      $port_scaler = $3;
      @port_list = split(/;/,$port_scaler);
    }
  }# if($found_entity == 1 && $found_end == 1)
  foreach $port (@port_list)
  {
      $port =~ s/#//g;
      if($port =~ /(.*)(:)(\s+)(\w+)(\s+)(.*)/)
      {
        $port_name = $1;
        $port_direction = $4;
        $port_direction =~ tr/A-Z/a-z/;
        $port_name =~ s/(\s+)//g;
        #print "port name = $port_name port direction = $port_direction\n";
        if($port_direction eq in)
        {
          if($in_port_scaler eq zero)
          {
            $in_port_scaler = $port_name;
          }
          else
          {
            $in_port_scaler = $in_port_scaler.",".$port_name;
          }
        }
        elsif($port_direction eq out)
        {
          if($out_port_scaler eq zero)
          {
            $out_port_scaler = $port_name;
          }
          else
          {
            $out_port_scaler = $out_port_scaler.",".$port_name;
          }
        }# elsif($port_direction eq out)
        else
        {
          print "ERROR!, port $port_name is of type '$port_direction' no constraints for it\n";
        }
      }# if($port =~ /(.*)(:)(\s+)(\w+)(\s+)(.*)/)
  }# foreach $port (@port_list)

  $in_port_scaler =~ s/\(//g;
  $in_port_scaler =~ s/\)//g;
  $out_port_scaler =~ s/\(//g;
  $out_port_scaler =~ s/\)//g;

  @in_port_vector = split(/,/,$in_port_scaler);
  @out_port_vector = split(/,/,$out_port_scaler);


  print out "#Auto Generated Constraints Template\n";
  print out "# Edit it to put real constraints.\n";
  print out "\n#Define I/O delays w.r.t clock Here\n";
  print out "#Keyword Value_as_a_fraction_of_Clk_Period port_name clk_name\n";
  print out "#Input Ports\n";

  foreach $ii (@in_port_vector)
  {
    if($ii ne $clk_name)
    {
      print out "set_input_delay \[expr 0\.7\*\$clk_period\] $ii -clock $clk_name -add_delay\n"
    }
  }# foreach $ii (@in_port_vector)

  print out "\n#Output Ports\n";

  foreach $ii (@out_port_vector)
  {
    print out "set_output_delay \[expr 0\.7\*\$clk_period\] $ii -clock $clk_name -add_delay\n"
  }# foreach $ii (@out_port_vector)
  print out "\n#Define false paths Here\n";
  print out "#set_false_path  -from clk -through sel -to dat_out\n\n";

  print out "\n#Define Virtual Clock to constraint input_to_output combi paths here\n";
  print out "#create_clock -name VCLK -period \$clk_period\n";
  print out "#set_false_path -from VCLK -to \$clk_name\n";
  print out "#set_false_path -from \$clk_name -to VCLK\n";


  print "INFO! output file 'constraints.tcl' generated\n";
